#!/bin/env python

# pyquery.py

# Please set or add the path of the Centrify Python modules to PYTHONPATH
# in order to run this script. For example:
#
# $ export PYTHONPATH=/usr/share/centrifydc/python/lib64
#

import argparse
import pycapi

def dumpObject(cdcObj, indent=""):
    for key in cdcObj:
        l = len(cdcObj[key])
        print("%s%s" % (indent, key), end=":")
        if l > 0:
            for i in range(l):
                print(cdcObj[key][i], end=" ")
        print("")

def dumpObjectList(cdcObjList, indent=""):
    l = len(cdcObjList)
    for i in range(l):
        print("%sObject %s:" % (indent, i))
        dumpObject(cdcObjList[i], "%s  " % (indent))

def dumpStringSet(cdcStringSet, indent=""):
    l = len(cdcStringSet)
    for i in range(l):
        print("%s%s" % (indent,cdcStringSet[i]))

parser = argparse.ArgumentParser(description="Demo the use of the pycapi module for query.")
parser.add_argument("type", choices=["user", "group", "dc", "dcs", "domain", "forest"], help="query type")
parser.add_argument("-n", "--name", help="name to be queried")

args = parser.parse_args()

s = pycapi.OpenSession()
if args.type == "user":
    cdcObj = pycapi.LookupObjectByName(s, pycapi.CDC_AD_CATEGORY_USER, args.name)
    dumpObject(cdcObj)
elif args.type == "group":
    cdcObj = pycapi.LookupObjectByName(s, pycapi.CDC_AD_CATEGORY_GROUP, args.name)
    dumpObject(cdcObj)
elif args.type == "dc":
    cdcObj = pycapi.GetDCInfo(s, args.name)
    dumpObject(cdcObj)
elif args.type == "dcs":
    cdcStringSet = pycapi.GetDomainControllers(s, args.name, pycapi.GETDC_FLAGS_GET_ALL | pycapi.GETDC_FLAGS_ANY_SITE)
    dumpStringSet(cdcStringSet)
elif args.type == "domain":
    cdcObjList = pycapi.GetDomainList(s, pycapi.GETDC_FLAGS_GET_ALL)
    dumpObjectList(cdcObjList)
elif args.type == "forest":
    cdcObjList = pycapi.GetForestList(s, pycapi.GETDC_FLAGS_GET_ALL)
    dumpObjectList(cdcObjList)

